# import json
from todx import fabric
# import os
# from collections import namedtuple
from todx import filehandler



todolist = fabric.TodoList("Science Stuff", ['science', 'tech'])
todolist.add_todo("To make a reactor")
todolist.add_todo("Start a reactor")
todolist.add_todo("Workk on a reactor", "✗")
todolist.add_todo("Research a reactor", "✓")

todolists = [todolist, todolist, todolist]

filehandler.save_file("data.json", todolists)
# print(jdump)

# with open("data.json", "w") as data_file:
#     data_file.write(jdump)

todolists2 = filehandler.load_file("data.json")

for tlist in todolists2:
    tlist.view_list()


# -*- coding: utf-8 -*- 
status_aliases_dict = {
    'v': '✓',
    'x': '✗'
}

def status_aliases(status):
    if status in status_aliases_dict:
        return status_aliases_dict[status]
    return status



# -*- coding: utf-8 -*- 
import fabric as fb
import appdirs

T1 = fb.Todo("Idiot am I", "o")

print(T1.content)
print(T1.status)

stk = fb.TodoList("Man at Arms", ["cool", "Awesome"])

stk.add_todo("Bang", "0")
stk.add_todo("Awesome stuff needs to be done")

stk.view_list()

print(appdirs.user_data_dir("todx"))


#!/usr/bin/env python3
# -*- coding: utf-8 -*- 
# import os
import argparse
import appdirs

from todx import fabric
from todx import filehandler
from todx import searcher
# from yesnoquery import query_yes_no as qyn

def main_command():
    # Load datafile and parse arguments
    app_data_file = appdirs.user_data_dir('todx') + '/data.json'

    todolists = filehandler.load_file(app_data_file)

    parser = argparse.ArgumentParser(description='A CLI ToDo App')
    parser.add_argument('-a', '--add',
                        nargs='*',
                        help='add a todo')
    parser.add_argument('-v', '--view',
                        default=None,
                        const='inbox',
                        nargs='?',
                        help='view the passed list by default views inbox')
    parser.add_argument('--all',
                        action='store_true',
                        help='view all the files')
    args = parser.parse_args()
    # print(args)


    if args.add != None:
        if len(args.add) == 0:
            print("Please provide at least one argument: content list-title[optional]  status[optional]")
        elif len(args.add) == 1:
            if len(todolists) == 0:
                todolists.append(fabric.TodoList("inbox"))
                todolists[0].add_todo(args.add[0])
            else:
                todolists[0].add_todo(args.add[0])
            todolists[0].view_list()
        # print("Added todo + ", args.add[0])
        
        elif len(args.add) == 2:
            index_list = searcher.find_list_index(args.add[1], todolists)
            if index_list >= 0:
                todolists[index_list].add_todo(args.add[0])
                todolists[index_list].view_list()
            else:
                newlist = fabric.TodoList(args.add[1])
                newlist.add_todo(args.add[0])
                todolists.append(newlist)
                newlist.view_list()
        
        elif len(args.add) == 3:
            index_list = searcher.find_list_index(args.add[1], todolists)
            if index_list >= 0:
                todolists[index_list].add_todo(args.add[0], args.add[2])
                todolists[index_list].view_list()
            else:
                newlist = fabric.TodoList(args.add[1])
                newlist.add_todo(args.add[0], args.add[2])
                todolists.append(newlist)
                newlist.view_list()
        
        else:
            print("Error: Too many Arguments (Tip: use quotes to surround todo content)")

    if args.view is not None:
        index_list = searcher.find_list_index(args.view, todolists)

        if index_list >= 0:
            todolists[index_list].view_list()
        else:
            print("List with title " + args.view + " could not be found!")
    
    if args.all is not False:
        for tlist in todolists:
            print()
            tlist.view_list()
    # Final cleanup and close
    filehandler.save_file(app_data_file, todolists)


# -*- coding: utf-8 -*- 
from todx.settings import status_aliases

class Todo:
    """
    A class to represent a single todo
    A todo has content and status, default status is blank
    """
    def __init__(self, content='', status=' '):
        self.__type__ = "Todo"
        self.content = content
        self.status = status
        # self._index = 0
    
    def __str__(self):
        return '[' + status_aliases(self.status) + '] ' + self.content


class TodoList():
    """
    A class to represent a List of todos.
    The main list is inventory, and the list has a title and tags
    """

    def __init__(self, title, tags=[]):
        """
        This function creates and initialises empty TodoList with title and tags
        """
        self.__type__ = "TodoList" # useful for json serialization
        self.title = title
        self.tags = tags
        self.inventory = [] # list of objects of class Todo 

    def add_tag(self, tag):
        self.tags.append(tag)

    def change_title(self, title):
        self.title = title

    def view_list(self):
        """
        View list's title tags and todos
        """
        print(self.title)
        if len(self.tags) != 0:
            print("Tags :", str(self.tags).strip('[]'))
        print()
        for item in self.inventory:
            print(item)

    def index_view(self):
        """
        View list's title tags and todos with indexes
        """
        print(self.title)
        if len(self.tags) != 0:
            print("Tags :", str(self.tags).strip('[]'))
        print()
        for i in range(len(self.inventory)):
            print(i + " ", self.inventory[i])
    
    def todo_view(self, index):
        """
        View a specific todo of given index
        """
        try:
            print(self.inventory[index])
        except IndexError:
            print("ERROR: The given index doesn't exist")

    def tag_index_view(self):
        """
        View only the index and tags of the list
        """
        print(self.title)
        if len(self.tags) != 0:
            print("Tags :", str(self.tags).strip('[]'))
        print()

    def add_todo(self, content, status=" "):
        self.inventory.append(Todo(content, status))
    
    def delete_todo(self, index):
        try:
            self.inventory.pop(index)
        except IndexError:
            print("ERROR: The given index doesn't exist")

    def change_status(self, index, status):
        try:
            self.inventory[index].status = status
        except IndexError:
            print("ERROR: The given index doesn't exist")



# -*- coding: utf-8 -*- 
import json
import os
import errno
from todx import fabric
# from collections import namedtuple

def jdefault(o):
    return o.__dict__

def list_decoder(objs):
    """
    Function to parse the json list of dictionary into objects of TodoList.
    """
    todolists = []
    for obj in objs:
        # print(obj)
        if '__type__' in obj and obj['__type__'] == 'TodoList':
            objT = fabric.TodoList(obj['title'], obj['tags'])
            for todo in obj['inventory']:
                objT.add_todo(todo['content'], todo['status'])
        else:
            objT = obj
        todolists.append(objT)
    return todolists

def load_file(filename):
    """
    Function to load datafiles, it returns a list of TodoList objects
    """
    if not os.path.exists(os.path.dirname(filename)):
        try:
            os.makedirs(os.path.dirname(filename))
        except OSError as exc: # Guard against race condition
            if exc.errno != errno.EEXIST:
                raise
        return []

    with open(filename) as data_file:
        read_data = data_file.read()
        return list_decoder(json.loads(read_data))

def save_file(filename, data):
    """
    Function to save data <list of TodoList Objects> to a json file in json format 
    """
    with open(filename, "w+") as data_file:
        jsondata = json.dumps(data, default=jdefault, indent=4)
        data_file.write(jsondata)

"""
I understand that my json data encoder and decoder are absolute crap However they work and thats what is great. I have borrowed code from about 4 stackoverflow answers and used some trial and error to get to this stage. I would love to refactor it, but I would love to do a lot of other things, so this remains as it is. Do not modify this unless you are brave enough (and crazy enough).
"""


# -*- coding: utf-8 -*- 
from fuzzywuzzy import fuzz
# from todx import fabric


def find_list_index(list_title, todolists):
    for i, tlist in enumerate(todolists):
        if fuzz.partial_ratio(list_title, tlist.title) > 80:
            return i
    return -1


# -*- coding: utf-8 -*- 
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="todx",
    version="0.0.6",
    author="xypnox",
    author_email="xypnox@gmail.com",
    description="A simple todo application",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/xypnox/todxpy",
    packages=setuptools.find_packages(),
    entry_points = {
        'console_scripts': ['todx=todx.main:main_command'],
    },
    install_requires = ['appdirs', 'fuzzywuzzy[speedup]'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
    ],
)
